@@ -1,12 +1,21 @@
-from selenium.webdriver.support import expected_conditions as EC
-from selenium.common.exceptions import NoAlertPresentException
+"""
+**_LIB BCFOX - DOC_**
+
+Args:
+    *DRIVER*: Para utitlizar o servico do _driver_ inicie a classe dele como no exemplo abaixo, todas as explicações das opções estão lá.
+        - driver_class = bcpkgfox.Instancedriver()
+
+    _SYSTEM_: Caso queira extrair PDF's, criar pastas, mostrar mensagens_box entre outros serviços do systema utilize o comando abaixo **diretamente**.
+        - bcpkgfox.System.{def}
+
+    _API_: Utilização de api's use o comando abaixo, **diretamente**
+        - bcpkgfox.invoke_api.{def}
+
+"""
+
 from selenium.webdriver.remote.webelement import WebElement
-from selenium.webdriver.support.ui import WebDriverWait
-from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.common.by import By
 from typing import Optional
-import multiprocessing
-import pyautogui
 import tkinter as tk
 import re
 
@@ -14,10 +23,8 @@ from .find_elements import *
 from .invoke_api import *
 from .get_driver import *
 from .system import *
-from pathlib import Path
 
 import os
-import sys
 
 dir_kit = "C:\\TMPIMGKIT\\LAST_IMG"
 
@@ -89,6 +96,20 @@ def create_dirs(specifics_dirs: Optional[list] = None, disable_print_response: b
         else:
             print(f" {DK_ORANGE}>{RESET} Todos os diretórios padrões presentes")
 
+def resource_path(relative_path):
+    """
+    Returns the absolute path of a resource, whether running normally or in a PyInstaller executable.
+
+    Args:
+        relative_path (str): Relative path of the resource.
+
+    Returns:
+        str: Absolute path of the resource.
+    """
+    try: base_path = sys._MEIPASS
+    except AttributeError: base_path = os.path.abspath(".")
+    return os.path.join(base_path, relative_path)
+
 def initialize_driver(extension_path: Optional[str] = None, captcha_name: Optional[str] = None, captcha_api_key: Optional[str] = None) -> WebElement:
 
     """ Passe somente o nome da pasta, no mesmo diretório da main """
@@ -241,16 +262,21 @@ def selectfox(elemento, method, key):
         select.select_by_value(key)
 
     if method == "text":
-        select.select_by_visible_text(key)
+        elements = WebDriverWait(elemento, 10).until(EC.visibility_of_all_elements_located((By.XPATH, '//option')))
+        for elm in elements:
+            if key.lower().strip() in elm.text.lower().strip():
+                select.select_by_visible_text(elm.text)
+                return
+        raise ModuleNotFoundError(f"Option {key} não encontrada")
 
     if method == "index":
         select.select_by_index(key)
 
-def pop_up_extract(text: bool = False, accept: bool = False, timeout: int = 10):
+def pop_up_extract(text: bool = False, accept: bool = False, timeout: int = 10, driver_instance: Optional[WebElement] = None):
     """ Identifica um pop-up simples extraindo o texto e aceitando ele também. \n
 
     - Como usar:
-        Chame a função e registrando ela em uma variável
+        Chame a função e registrando ela em uma variável, and if you iniciate a driver without the library pass the driver variable.
 
     - Exemplo:
         text = bc.pop_up_extract(text:True, accept:True, timeout=5)
@@ -260,13 +286,14 @@ def pop_up_extract(text: bool = False, accept: bool = False, timeout: int = 10):
     global driver
     extract_text = None
 
+    if not driver_instance: driver_instance = driver
     if timeout == 0:
         timeout = float("inf")
 
     attempts = 0
     while attempts < timeout:
         try:
-            jan = driver.switch_to.alert
+            jan = driver_instance.switch_to.alert
 
             if text == True:
                 extract_text = jan.text
@@ -278,11 +305,13 @@ def pop_up_extract(text: bool = False, accept: bool = False, timeout: int = 10):
                 return extract_text
             return
 
-        except:
+        except Exception as e:
+            last_exception = e
+            print(e)
             attempts += 1
             time.sleep(0.8)
 
-    raise ValueError("Pop-up não encontrado")
+    raise ValueError("Pop-up não encontrado") from last_exception
 
 def cpf_or_cnpj(numero: str) -> str:
     """
@@ -304,8 +333,15 @@ def cpf_or_cnpj(numero: str) -> str:
 
     return "Inválido"
 
+janela = None
 @staticmethod
-def mostrar_mensagem(mensagem, tamanho_fonte=12, negrito=False):
+def mostrar_mensagem(mensagem, tamanho_fonte=12, negrito=False, button: Optional[bool] = True):
+
+    global janela
+
+    try: fechar_janela(janela)
+    except: pass
+
     root = tk.Tk()
     root.withdraw()
 
@@ -323,17 +359,26 @@ def mostrar_mensagem(mensagem, tamanho_fonte=12, negrito=False):
     estilo_fonte = ("Helvetica", tamanho_fonte, "bold" if negrito else "normal")
 
     label = tk.Label(janela, text=mensagem, bg="white", fg="black",
-                        font=estilo_fonte, wraplength=380, justify="center")
+                     font=estilo_fonte, wraplength=380, justify="center")
     label.pack(expand=True, padx=20, pady=20)
 
-    tk.Button(janela, text="OK", command=lambda: (janela.destroy(), root.destroy()),
-                width=10, font=("Helvetica", 10)).pack(pady=10)
+    if button:
+        tk.Button(janela, text="OK", command=lambda: (janela.destroy(), root.destroy()),
+                  width=10, font=("Helvetica", 10)).pack(pady=10)
 
     janela.grab_set()
     janela.focus_set()
-    janela.wait_window()
+    if button: janela.wait_window()
+    else: root.mainloop()
+    return janela
+
+def fechar_janela(janela_=None):
+    global janela
+    if janela_ == None: janela.destroy()
+    else: janela_.destroy()
 
 def move_mouse_smoothly(element, click=False):
+    import pyautogui
     cordenadas_botao_certificado = element.location
     x = cordenadas_botao_certificado["x"]+140
     y = cordenadas_botao_certificado["y"]+112
@@ -373,5 +418,6 @@ def move_mouse_smoothly(element, click=False):
 RESET, GR, ORANGE, DK_ORANGE = "\033[0m", "\033[38;5;34m", "\033[38;5;214m", "\033[38;5;130m"
 result = subprocess.run(['pip', 'show', "bcpkgfox"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 version_line = next((line for line in result.stdout.decode().splitlines() if line.startswith('Version:')), None)
-print(f"\n\n{ORANGE}Biblioteca BCFOX importada - {re.sub(r"[^0-9.b]", "", version_line)}{RESET}")
+try: print(f"\n\n{ORANGE}Biblioteca BCFOX importada - {re.sub(r"[^0-9.b]", "", version_line)}{RESET}")
+except: pass
 create_dirs()
\ No newline at end of file
