@@ -1,3 +1,4 @@
+from selenium.common.exceptions import ElementClickInterceptedException
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.remote.webelement import WebElement
 from selenium.webdriver.support.ui import WebDriverWait
@@ -9,15 +10,43 @@ from selenium import webdriver
 
 from typing import Optional
 
+import random
 import time
 import math
-import random
-import pyautogui
 
 class Elements:
     def __init__(self, driver):
         self.driver = driver
 
+    def click_with_wait(
+        self,
+        by: str,
+        value: str,
+        timeout: Optional[int] = 10,
+        parent: Optional[WebElement] = None
+    ) -> WebElement:
+        """ The diference of this to 'find_element_with_wait is bc this dont will return a 'intercepted click' error """
+
+        if "css" in by:
+            by = By.CSS_SELECTOR
+
+        if parent is None: parent = self.driver
+        attp = 0
+        while attp < timeout:
+            try:
+                return WebDriverWait(parent, 1).until(
+                    EC.presence_of_element_located((by, value))
+                ).click()
+            except ElementClickInterceptedException as e:
+                ultima_exception = e
+                time.sleep(1)
+                attp += 1
+
+            except Exception as e:
+                ultima_exception = e
+                attp += 1
+        raise SystemError(ultima_exception)
+
     def find_element_with_wait(
         self,
         by: str,
@@ -50,6 +79,23 @@ class Elements:
             EC.presence_of_all_elements_located((by, value))
         )
 
+    def script_console_with_wait(self,
+        script: str,
+        timeout: Optional[int] = 10
+    ) -> None:
+
+        attempts = 0
+        if timeout == 0: timeout = float("inf")
+        while attempts <= timeout:
+            try:
+                self.driver.execute_script(script)
+                return
+            except Exception as e:
+                last_e = e
+                attempts += 1
+                time.sleep(1)
+        raise TimeoutError() from last_e
+
     def wait_for_element_be_clickable(
         self,
         by: str,
@@ -67,32 +113,35 @@ class Elements:
         )
 
     def move_to_image(self,
-        imagens: [str | list],
+        imagens: str | list[str],
         click_on_final: Optional[bool] = False,
         trail: Optional[bool] = False,
         tolerancia: Optional[float] = 0.8,
         timeout: Optional[int] = 10,
+        verify: Optional[bool] = False,
         repeat: Optional[bool] = False
     ) -> str:
         """
         Move o mouse até o centro de uma imagem na tela e clica, se necessário.
 
-        Args:
+        Attributes:
             imagens (list) : Caminho da imagem a ser localizada.
             click_on_final (bool): Se True, realiza um clique ao final.
-            trail: O mouse faz todo o caminho até o botão.
-            tolerancia (float): Tolerância para a comparação da imagem (opcional, valor padrão: 1).
+            trail (bool): O mouse faz todo o caminho até o botão.
+            tolerancia (float): Tolerância para a comparação da imagem (opcional, valor padrão: 0.8).
             timeout (int): Tempo máximo (em segundos) para procurar a imagem antes de desistir (opcional, valor padrão: 10).
+            verify (bool): Apenas verifica se o elemento está na tela, interagir.
             repeat (bool): Se for True ele ignora o timeout, e fica tentando encontrar a imagem até conseguir, sem gerar erro
 
-        Exemplo:
-            caminho_imagem = 'C:\\User\\Caminho\\exemplo.png'
+        Example:
+            caminho_imagem = 'C:\\User\\Caminho\\exemplo.png'\n
             move_to_image(caminho_imagem, click_on_final=True, tolerancia=0.9, timeout=30)
-
-        Nota:
-            Recomenda-se colocar a imagem na mesma pasta do arquivo MAIN para evitar problemas ao gerar o executável com pyinstaller e rodar em outras máquinas.
+        \n\n
+        Note: Recomenda-se criar uma pasta "imagens" no mesmo diretório da main e identificar as imagens com resource_path para não bugar o pyinstaller
         """
 
+        import pyautogui
+
         if isinstance(imagens, str):
             imagens = [imagens]
 
@@ -107,23 +156,19 @@ class Elements:
                     try:
                         localizacao = pyautogui.locateOnScreen(imagem, confidence=tolerancia)
                         break
-                    except:
-                        localizacao = None
-                        continue
+                    except: continue
 
-                if localizacao is not None:
+                if localizacao:
                     x = localizacao.left + round(localizacao.width / 2)
                     y = localizacao.top + round(localizacao.height / 2)
 
                     if trail: self.move_mouse_smoothly(pure=[x,y])
-                    else: pyautogui.moveTo(x, y)
+                    else:
+                        if not verify: pyautogui.moveTo(x, y)
 
-                    if click_on_final:
-                        pyautogui.click()
+                    if click_on_final: pyautogui.click()
                     return localizacao
 
-                else: raise FileNotFoundError()
-
             except Exception as e:
                 attempts += 1
                 ultima_excecao = e
@@ -146,7 +191,9 @@ class Elements:
     def move_mouse_smoothly(self,
         element: Optional[WebElement] = None,
         pure: Optional[list]=None,
-        click: Optional[bool]=False
+        click: Optional[bool]=False,
+        x_adittional: Optional[int] = 140,
+        y_adittional: Optional[int] = 120
         ) -> None:
         """
         Element: Web element of selenium
@@ -158,8 +205,8 @@ class Elements:
             y = pure[1]
         else:
             cordenadas_botao_certificado = element.location
-            x = cordenadas_botao_certificado["x"]+140
-            y = cordenadas_botao_certificado["y"]+112
+            x = cordenadas_botao_certificado["x"]+x_adittional
+            y = cordenadas_botao_certificado["y"]+y_adittional
 
         actual_position = pyautogui.position()
         mouse_x = actual_position[0]
@@ -238,7 +285,7 @@ class Elements:
                 break
 
         attempt = 0
-        timeout = float(inf) if timeout == 0 else timeout
+        timeout = float("inf") if timeout == 0 else timeout
 
         if type == "text":
             while attempt < timeout:
@@ -426,6 +473,7 @@ def move_to_image(imagens, click_on_final=False, tolerancia=1, timeout=10, repea
     Nota:
         Recomenda-se colocar a imagem na mesma pasta do arquivo MAIN para evitar problemas ao gerar o executável com pyinstaller e rodar em outras máquinas.
     """
+    import pyautogui
 
     if isinstance(imagens, str):
         imagens = [imagens]
